unit IonsO;
{ Describes the dynamics of the ionic concentrations for Ko and Cli
  and calculates the reversal potentials VK and VCl. }

interface
uses
  Classes,Forms,
  t_dtO,typeSynParsO,MathMyO,Unit26,{Init,}typeNrnParsO,ConnectionsO,NaKpump;

const  e0=0.02663 {V};

type
  {==================================================================}
  IonsProperties =record
    K_0,Cl_0,Na_0,K_fix,Cl_fix,Cl_normal,Na_normal,
    Ki_0,Cl_inf,Clo_0,Nai_alpha,Ko_alpha,K12,
    gam,beta,gL_K,gL_Cl,gL_Na,q_Na,
    q_Ca,INCX_max,Cao_0,
    dVAHP,dVK,dVKDr,dVKM,dVKD,dVNa,dVGABA,dVGABB,dVL,dVLd,
    Bmax,k1,k1N, K_bath0, Ikcc2_max,Inkcc1_max0, Vhalf,
    Nao_0,  gg_kl, gg_Nal, INaKpump_max0,INaKpump0,
    Diff_coe, D_2d_coe, al_O2, Diff_O2, O2_bath,
    Contribution,VGABA0,HCO3in,HCO3out,Cl_bound,
    tau_Vol,Lw_Vol,pio_pii_0
                                                          :double;
    If_Glia,If_Diffusion,If_K_fixed,If_Cl_fixed,If_O2,
    If_Volume,
    NaKpump_Item,KCC2_Item,VL_Item,Vol_Item               :integer;
  end;

  IonsVariables =record
    Ki,K,Cl,Na,O2,Clo,Nao,Ca,
    VCl,VK,VNa,VL,VGABA,VCa,Vol,
    INaKpump,INaCaX,
    B,I_Glia,I_Diff,
    K_bath_of_x,Inkcc1_max_of_x,INaKpump_max_of_x
                                                          :double;
  end;
  IonsVarToView =record
    IK_,IK_leak_,ICl_leak_,INa_leak_,
    ICl_,IKCC2_,INKCC1_,I_K_Na_Cl_,IGABA_,IGABB_,
    INa_glu_,IK_glu_,ICa_glu_
                                                          :double;
  end;

  TIons = class
  private
  protected
  public
    iP,                    // iP - index of population [1..4],
    iProp  :integer;       // iProp - index of its properties [1..3]
    iii  :text;
    {Variables:}
    IV      :IonsVariables;
    {Variables to plot and write}
    VW      :IonsVarToView;
    {Functions:}
    {*******************************************}
    procedure ReversalsFromConcentrations;
    procedure InitialConditions(iPop_ :integer; var NP_ :NeuronProperties);
    function GlialBuffer(dt_ :double) :double;
    function GlialBuffer2(dt_ :double) :double;
    function NaKpump(Umean{V} :double) :double;
    procedure Integrate(dt_,Umean{V},Udmean{V},IK_tot,INa_ind{mA/cm^2}{mA/cm^2},
                          gAMPA_,gNMDA_,gGABA_,gGABB_{mS/cm^2},
                          nu{Hz},Lapl_K_{mM/mum2} :double);
    procedure Draw;
    procedure Draw2;
    procedure InitiateWriting;
    procedure Writing;

    constructor Create;
    procedure EquateFrom(AIons :TIons);
    procedure EquateTo(AIons :TIons);
  end;
  {==================================================================}
  procedure ParamsFromForm26;

var
  {Properties:}
  IonsP                                    :array[1..3] of IonsProperties;

implementation

var
    NMDA_K,NMDA_Na,NMDA_Ca,AMPA_Ca
                                          :double;

constructor TIons.Create;
begin
  inherited Create;
end;

procedure TIons.EquateFrom(AIons :TIons);
begin
  { Properties }
  iP :=AIons.iP ;
  iProp :=AIons.iProp ;
  { Variables }
  IV :=AIons.IV ;
end;

procedure TIons.EquateTo(AIons :TIons);
begin
  { Properties }
  AIons.iP :=iP;
  AIons.iProp :=iProp;
  { Variables }
  AIons.IV :=IV ;
end;

procedure ParamsFromForm26;
var iPop :integer;
begin
  for iPop:=1 to 3 do with IonsP[iPop] do begin
    { K and Cl concentrations }
    k1        :=Form26.DDSpinEdit4.Value/1000; //0.0008{1/ms} in [Bazhenov,Timofeev,...,2004]
    gam       :=Form26.DDSpinEdit5.Value/1000; {mM*cm2/(muA*ms),   gam=S/F/vol=k/F; 10/96484 in [Bazhenov] but 4.4e-5 in [Barreto, Cressman 2011]; my estimation - 0.72e-3}
    beta      :=Form26.DDSpinEdit6.Value; //7 // volume ratio of extracell. compartment to the surface area, 7 in [Cressman], 1/0.5 in [Bazhenov];
    K_fix :=Form26.DDSpinEdit22.Value;  { Ko fixed, mM }
    Cl_fix:=Form26.DDSpinEdit23.Value;  { Cli fixed, mM }
    If_K_fixed  :=IfTrue(Form26.CheckBox2.Checked);
    If_Cl_fixed :=IfTrue(Form26.CheckBox3.Checked);
    If_O2       :=IfTrue(Form26.CheckBox5.Checked);
    If_Volume   :=IfTrue(Form26.CheckBox6.Checked);
    VL_Item     :=Form26.RadioGroup3.ItemIndex;
    Form26.DDSpinEdit4.Enabled:=Form26.CheckBox1.Checked;
    Form26.DDSpinEdit22.Enabled:=Form26.CheckBox2.Checked;
    Form26.DDSpinEdit23.Enabled:=Form26.CheckBox3.Checked;
    Form26.DDSpinEdit30.Enabled:=Form26.CheckBox5.Checked;
    Form26.DDSpinEdit33.Enabled:=Form26.CheckBox5.Checked;
    Form26.DDSpinEdit35.Enabled:=Form26.CheckBox5.Checked;
    Form26.DDSpinEdit9.Enabled:=not(Form26.CheckBox2.Checked);
    Form26.DDSpinEdit10.Enabled:=not(Form26.CheckBox3.Checked);
    Form26.DDSpinEdit3.Enabled:=not(Form26.CheckBox3.Checked);
    Form26.DDSpinEdit12.Enabled:=(Form26.CheckBox4.Checked);
    Form26.DDSpinEdit13.Enabled:=(Form26.CheckBox4.Checked);
    Form26.DDSpinEdit38.Enabled:=(Form26.CheckBox4.Checked);
    Form26.DDSpinEdit8.Enabled:=(NPop>=3);
    Form26.DDSpinEdit18.Enabled:=(NPop>=3);
    Form26.DDSpinEdit19.Enabled:=(NPop>=3);
    Form26.DDSpinEdit25.Enabled:=(NPop>=3);
    Form26.DDSpinEdit45.Enabled:=(NPop>=3);
    Form26.DDSpinEdit47.Enabled:=(NPop>=3);
    //Clo:=Form26.DDSpinEdit16.Value; //130 {mM};
    HCO3in :=Form26.DDSpinEdit16.Value; //16 {mM};      { Doyon et al. 2011 }
    K_bath0:=Form26.DDSpinEdit13.Value;
    { GABA }
    case iPop of
    1: Cl_bound:=Form26.DDSpinEdit24.Value;    // mM  {Glykys et al.K_0 2014}
    2: Cl_bound:=Form26.DDSpinEdit27.Value;
    3: Cl_bound:=Form26.DDSpinEdit25.Value;
    end;
    { Glia }
    If_Glia:=IfTrue(Form26.CheckBox1.Checked);
    { NaK-pump }
    INaKpump_max0:=Form26.DDSpinEdit20.Value; //20 {muA/cm^2};
    NaKpump_Item :=Form26.ComboBox1.ItemIndex;
    Vol_Item     :=Form26.ComboBox2.ItemIndex;
    K12          :=Form26.DDSpinEdit36.Value; // mM
    { O2 }
    al_O2  :=Form26.DDSpinEdit30.Value; //5.3 {g/mol} {Wei JNPh2014};
    Diff_O2:=Form26.DDSpinEdit33.Value/1000; //0.17e-3 {1/ms} {Wei JNPh2014};
    O2_bath:=Form26.DDSpinEdit35.Value; //32 {mg/l} {Wei JNPh2014};
    { KCC2 }
    KCC2_Item   :=Form26.RadioGroup1.ItemIndex;
    { NKCC1 Dronne 2006 }
    case iPop of
    1: Inkcc1_max0:=Form26.DDSpinEdit11.Value;      // muA/cm^2
    2: Inkcc1_max0:=Form26.DDSpinEdit7.Value;       // muA/cm^2
    3: Inkcc1_max0:=Form26.DDSpinEdit8.Value;       // muA/cm^2
    end;
    { KCC2 Doyon }
    case iPop of
    1: Ikcc2_max:=Form26.DDSpinEdit14.Value;     // muA/cm^2
    2: Ikcc2_max:=Form26.DDSpinEdit26.Value;     // muA/cm^2
    3: Ikcc2_max:=Form26.DDSpinEdit18.Value;     // muA/cm^2
    end;
    case KCC2_Item of
    0: {Doyon}    begin
                    Form26.DDSpinEdit14.Enabled:=true;
                    Form26.DDSpinEdit18.Enabled:=(NPop>=3);
                    Form26.DDSpinEdit26.Enabled:=true;
                  end;
    1: {Cressman}    begin
                    Form26.DDSpinEdit14.Enabled:=true;
                    Form26.DDSpinEdit18.Enabled:=(NPop>=3);
                    Form26.DDSpinEdit26.Enabled:=true;
                  end;
    end;
    case Vol_Item of
    0: {Wei}      begin
                    Form26.DDSpinEdit37.Enabled:=true;
                    Form26.DDSpinEdit15.Enabled:=false;
                  end;
    1: {OstbyEinvoll} begin
                    Form26.DDSpinEdit37.Enabled:=false;
                    Form26.DDSpinEdit15.Enabled:=true;
                  end;
    2: {Ostby&me} begin
                    Form26.DDSpinEdit37.Enabled:=true;
                    Form26.DDSpinEdit15.Enabled:=true;
                  end;
    3: {steady osmos} begin
                    Form26.DDSpinEdit37.Enabled:=false;
                    Form26.DDSpinEdit15.Enabled:=false;
                  end;
    end;
    { Diffusion }
    If_Diffusion:=IfTrue(Form26.CheckBox4.Checked);
    Diff_coe:=Form26.DDSpinEdit12.Value/1000;   {1/ms}
    D_2d_coe:=Form26.DDSpinEdit38.Value;        {mum^2/ms}
    { Contribution to K of Pop 2,3 }
    case iPop of
      1: begin
           if NPop>2 then
              Contribution:=1-Form26.DDSpinEdit19.Value-Form26.DDSpinEdit28.Value
           else if NPop=2 then
              Contribution:=1-Form26.DDSpinEdit28.Value
           else
              Contribution:=1;
         end;
      2: Contribution:=Form26.DDSpinEdit28.Value;
      3: Contribution:=Form26.DDSpinEdit19.Value;
    end;
    { Volume }
    tau_Vol:=Form26.DDSpinEdit37.Value; { [Ullah et al. 2015] }
    Lw_Vol :=Form26.DDSpinEdit15.Value; {1/ms/mM} {Suppl. of [Ostby et al. 2009]}
    { Ca }
    INCX_max :=Form26.DDSpinEdit42.Value; // INCX_max, muA/cm^2    //time scale of Ca extrusion by transporters, ms
    case iPop of
    1: q_Ca   :=Form26.DDSpinEdit41.Value; // charge of a spike =0.01muC/cm^2=0.1mA/cm^2 * 1ms
    2: q_Ca   :=Form26.DDSpinEdit46.Value; // charge of a spike =0.01muC/cm^2=0.1mA/cm^2 * 1ms
    3: q_Ca   :=Form26.DDSpinEdit47.Value; // charge of a spike =0.01muC/cm^2=0.1mA/cm^2 * 1ms
    end;
  end;
end;

{******************************************************************************}
procedure TIons.ReversalsFromConcentrations;
var Nao_,Clo_,HCO3out_,Cao_,Cl_bound_ :double;
begin
  { Contra-side concentrations }
  Nao_     :=IV.Nao;//(IonsP[iProp].Nao0 - IonsP[iProp].beta*(IV.Na-IonsP[iProp].Na_normal))/IV.Vol; { VolumeEquations.doc, (v11') }
  Clo_     :=IV.Clo;//(IonsP[iProp].Clo0 - IonsP[iProp].beta*(IV.Cl-IonsP[iProp].Cl_normal))/IV.Vol; { VolumeEquations.doc, (v12') }
  Cao_     :=IonsP[iProp].Cao_0;
  HCO3out_ :=IonsP[iProp].HCO3out{/IV.Vol};                                      //commented on 15.07.2020        { VolumeEquations.doc, (v13) }
  Cl_bound_:=IonsP[iProp].Cl_bound;
  { Reversal potentials }
  IV.VCl  :=e0*ln(max(0.01,IV.Cl{-Cl_bound_})/Clo_);    {V}     { p.160 }       //commented cl_bound on 11.05.2018
  IV.VK   :=e0*ln(IV.K  /IV.Ki);     {V}
  IV.VNa  :=e0*ln(Nao_/IV.Na);     {V}
  IV.VCa  :=e0*ln(Cao_/IV.Ca);     {V}
  IV.VL   :=(IonsP[iProp].gL_Cl *IV.VCl
           + IonsP[iProp].gL_K  *IV.VK
           + IonsP[iProp].gL_Na *IV.VNa
           - IonsP[iProp].INaKpump0/1000{mA/cm^2})                                // added on 3.10.19 // sign was corrected on 14.07.2020
           /(IonsP[iProp].gL_Cl + IonsP[iProp].gL_K + IonsP[iProp].gL_Na);      {V}
  //IV.VL   :=e0*ln((IV.K+0.085*Nao+0.1*IV.Cl)/(Ki+0.085*IV.Na+0.1*Clo)); {V}    { [Bazhenov 2004] }
  //IV.VL   :=e0*ln((IV.K+0.065*IonsP[iProp].Nao+0.6*(IV.Cl-IonsP[iProp].Cl_bound))/(IonsP[iProp].Ki+0.065*IV.Na+0.6*IonsP[iProp].Clo)); {V}    { [Ullah 2009] }
  IV.VGABA:=e0*ln((IV.Cl-Cl_bound_ + IonsP[iProp].HCO3in/4)
                 /(Clo_ + HCO3out_/4)); {V}      { Doyon et al. 2011 }
end;

{******************************************************************************}
procedure TIons.InitialConditions(iPop_ :integer; var NP_ :NeuronProperties);
begin
  { iProp - type of IonsP[1..3]; iP - index of Ions[1..4]. Exclusion: Ions[4] has properties of IonsP[2]. }
  iP:=iPop_;
  iProp:=iP;  if iP>3 then iProp:=2-(IP mod 2);
  { Parameters }
  ParamsFromForm26;
  with IonsP[iProp] do begin
    Cl_inf :=5        {mM};
    Clo_0  :=130      {mM};
    HCO3out:=24       {mM};     { Amakhin et al. }
    Nao_0  :=125.90;  {mM}
    Cao_0  :=2;       {mM}      { Jones and Keep, 1987 }
    Ki_0   :=129.34;  {mM}
    Cl_normal:=Form26.DDSpinEdit39.Value;  { Cli normal, mM }
    Na_normal:=Form26.DDSpinEdit40.Value;  { Nai normal, mM }
    K_0 :=Form26.DDSpinEdit9.Value;        { Ko initial, mM }
    case iP of
    1: begin
         Cl_0:=Form26.DDSpinEdit10.Value;  { Cli initial, mM }
         Na_0:=Form26.DDSpinEdit29.Value;  { Nai initial, mM }
       end;
    2,4: begin
         Cl_0:=Form26.DDSpinEdit31.Value;  { Cli initial, mM }
         Na_0:=Form26.DDSpinEdit32.Value;  { Nai initial, mM }
       end;
    3: begin
         Cl_0:=Form26.DDSpinEdit3.Value;   { Cli initial, mM }
         Na_0:=Form26.DDSpinEdit34.Value;  { Nai initial, mM }
       end;
    end;
    gL_K      :=Form26.DDSpinEdit1.Value;  // 0.035  {mS/cm^2};
    gL_Cl     :=Form26.DDSpinEdit2.Value;  // 0.01   {mS/cm^2};
    gL_Na     :=Form26.DDSpinEdit21.Value; // 0.02; {mS/cm^2}
    q_Na      :=Form26.DDSpinEdit17.Value; // charge of a spike = 0.1muC/cm^2=0.1mA/cm^2 * 1ms

    gg_kl     :=0.042; {mS/cm^2}
    gg_Nal    :=0.022; {mS/cm^2}

    NMDA_K    :=Form26.DDSpinEdit43.Value;
    NMDA_Na   :=Form26.DDSpinEdit44.Value;
    NMDA_Ca   :=Form26.DDSpinEdit45.Value;
    AMPA_Ca   :=Form26.DDSpinEdit48.Value;
    //F:=96489 {C/mol}; F:=96 489 000 {muA*ms/mM/cm^2};
    Bmax:=500{mM};
    k1N:=1.1;
    Vhalf:=40;     // mV
    { Initial conditions }
    IV.K_bath_of_x      :=K_bath0;                    //introduced on 30.07.2020
    IV.Inkcc1_max_of_x  :=Inkcc1_max0;                //introduced on 30.07.2020
    IV.INaKpump_max_of_x:=INaKpump_max0;              //introduced on 10.09.2020
    IV.B:=Bmax;
    IV.Ki:=Ki_0;
    IV.K :=K_0;   if If_K_fixed=1  then IV.K :=K_fix;                            { K fixed, mM }
    IV.Cl:=Cl_0;  if If_Cl_fixed=1 then begin IV.Cl:=Cl_fix; IV.Clo:=Clo_0; end; { Cl fixed, mM }
    IV.Na:=Na_0;
    IV.Ca:=0.0001{mM};
    IV.Clo:=Clo_0;
    IV.Nao:=Nao_0;
    IV.O2:=O2_bath;
    IV.Vol:=1;
    ReversalsFromConcentrations;
    { Initiate Na-K pump }
    Initiate_NaKpump_SmithCrampin;
    Nai_alpha:=20 {mM};
    Ko_alpha:=2.5 {mM};
    IV.INaKpump:=NaKpump(NP_.Vrest);
    INaKpump0:=IV.INaKpump;
    pio_pii_0:= IV.K  + IV.Clo + IV.Nao + IonsP[iProp].HCO3out/IV.Vol
               -IV.Ki - IV.Cl  - IV.Na  - IonsP[iProp].HCO3in;                    {mM}
    { Set parameters for Bazhenov model }
    if NP_.HH_type='Bazhenov' then begin
       NP_.gLd :=gL_Cl+gL_K+gL_Na;
       NP_.gL  :=gg_kl+gg_Nal;
       NP_.l:=2*NP_.gLd*0.000165{cm^2}*10000{Ohm};
       NP_.VLd :=(gL_Cl*IV.VCl+gL_K*IV.VK+gL_Na*IV.VNa)/(gL_Cl+gL_K+gL_Na);
       NP_.VL  :=(gg_kl*IV.VK+gg_Nal*IV.VNa)/(gg_kl+gg_Nal);
       NP_.ro:=165*NP_.gLd/NP_.gL;
       NP_.NaKpumpS:=IV.INaKpump/1000; {mA/cm^2}
       NP_.NaKpumpD:=IV.INaKpump/1000; {mA/cm^2}
    end;
    { Residual between reversals of specific K-channels and pure K-ions. }
    dVAHP :=NP_.VAHP -IV.VK;
    dVK   :=NP_.VK   -IV.VK;
    dVKDr :=NP_.VKDr -IV.VK;
    dVKM  :=NP_.VKM  -IV.VK;
    dVKD  :=NP_.VKD  -IV.VK;
    dVNa  :=NP_.VNa  -IV.VNa;
    dVL   :=NP_.VL   -IV.VL;
    dVL   :=NP_.VLd  -IV.VL;
    { GABA-A and GABA-B }
    VGABA0:=IV.VGABA;
{    for i:=1 to NSyn do  with Syn[i].SP do if IfBlock<>1 then begin
        if (OwnerOfSynapse(Name)=iP) then begin
            if (Name[1]+Name[2]+Name[3]+Name[4]='GABA') or
               (Name[1]+Name[2]+Name[3]+Name[4]='GAB3') then begin
               dVGABA :=Syn[i].SP.Vrev - IV.VCl;
            end else
            if (Name[1]+Name[2]+Name[3]+Name[4]='GABB') then begin
               dVGABB :=Syn[i].SP.Vrev - IV.VCl;
            end;
        end;
    end;}
  end;
  { Ca }
  NP_.tauCa :=8888888;  //  Exclude the term in CondBasedO.pas
end;

function TIons.GlialBuffer(dt_ :double) :double;
{ Krishnan and Bazhenov 2011 }
var k2 :double;
begin
  GlialBuffer:=0;
  with IonsP[iProp] do begin
    if If_Glia<>1 then exit;
    case KCC2_Item of
    0,1: begin {Bazhenov KCC2 approximation [Krishnan 2015]}
           k2:=k1/(1+dexp(-(IV.K-15{mM})/1.15{mM}));
           {***********************************************}
           IV.B:=IV.B + dt_*1000*(k1*(Bmax-IV.B) - k2*IV.B);        {mM}
           {***********************************************}
           GlialBuffer:=         k1*(Bmax-IV.B)/k1N - k2*IV.B;     {mM/ms}
         end;
    2:   begin {[Wei 2014]}
           GlialBuffer:=Bmax*k1/(1+dexp((18{mM}-IV.K)/2.5));       {mM/ms}
         end;
    end;
  end;
end;

function TIons.GlialBuffer2(dt_ :double) :double;
{ Krishnan and Bazhenov 2011 }
var k2,G_ :double;
begin
  GlialBuffer2:=0;
  with IonsP[iProp] do begin
    if If_Glia<>1 then exit;
    case KCC2_Item of
    0,1: begin {Bazhenov KCC2 approximation [Krishnan 2015]}
           k2:=k1/(1+dexp(-(IV.K-15{mM})/1.15{mM}));
           G_:=k1*(Bmax-IV.B) - k2*IV.B;
           {**********************************************}
           IV.B:=IV.B + dt_*1000*G_;                                {mM}
           {**********************************************}
           GlialBuffer2:=G_;                                        {mM/ms}
         end;
    2:   begin {[Wei 2014]}
           GlialBuffer2:=Bmax*k1/(1+dexp((18{mM}-IV.K)/2.5));       {mM/ms}
         end;
    end;
  end;
end;

function TIons.NaKpump(Umean{V} :double) :double;
begin
  with IonsP[iProp] do begin
  case NaKpump_Item of
  0: begin { Bazhenov 2004; Kager 2000 }
       NaKpump:=IV.INaKpump_max_of_x/sqr(1+Ko_alpha/IV.K)/step(1+Nai_alpha/IV.Na,3); {muA/cm^2}
     end;
  1: begin { Cressman 2009; Wei 2014 }
       NaKpump:=IV.INaKpump_max_of_x/   (1+dexp(K12-IV.K))/  (1+dexp((25-IV.Na)/3))
                                    /   (1+dexp((20-IV.O2)/3));                      {muA/cm^2}
     end;
  2: begin { Smith and Crampin // Biophisics and Mol. Biol. 2004 }
       NaKpump:=IV.INaKpump_max_of_x * NaKpump_SmithCrampin(Umean,IV.Nao,IV.Na,IV.K,IV.Ki); {muA/cm^2}
     end;
  end;
  end;
end;

{******************************************************************************}
procedure TIons.Integrate(dt_,Umean{V},Udmean{V},IK_tot,INa_ind{mA/cm^2},
                            gAMPA_,gNMDA_,gGABA_,gGABB_{mS/cm^2},
                            nu{Hz},Lapl_K_{mM/mum2} :double);
{******************************************************************************}
var
   tau_Cl_,dKi_dt,dK_dt,dCl_dt,dNa_dt,dO2_dt,dVol_dt,dClo_dt,dNao_dt,dCa_dt,
   targetVol,pio,pii,pio_pii,WaterInflux,v_,tau_Ca_,
   G_,Diff_K,Diff_Cl,Diff_Na
                                                                      :double;
begin
  with IonsP[iProp] do begin
  VW.IK_leak_:=                 gL_K   *(Umean -IV.VK) *1e3;  {muA/cm^2}
  VW.ICl_leak_ :=                 gL_Cl  *(Umean -IV.VCl)*1e3;  {muA/cm^2}
  VW.INa_leak_ :=                 gL_Na  *(Umean -IV.VNa)*1e3;  {muA/cm^2}
  VW.IGABA_:=                     gGABA_ *(Umean -IV.VCl)*1e3;  {muA/cm^2} // changed Udmean to Umean on 05.09.2019
  VW.IGABB_:=                     gGABB_ *(Umean -IV.VK) *1e3;  {muA/cm^2} // introduced on 05.06.2020
  VW.IK_glu_   := NMDA_K *(gAMPA_+gNMDA_)*(Udmean-IV.VK) *1e3;  {muA/cm^2} // [p.166, Mayer&Westbrook1987]
  VW.INa_glu_  := NMDA_Na*(gAMPA_+gNMDA_)*(Udmean-IV.VNa)*1e3;  {muA/cm^2} // [--//--]
  VW.ICa_glu_  :=(NMDA_Ca*gNMDA_+AMPA_Ca*gAMPA_)
                                              *(Udmean- 0.020)*1e3;  {muA/cm^2} // [--//--]
  { Na-K pump }
  IV.INaKpump:=NaKpump(Umean);
  VW.IK_  := VW.IK_leak_ + VW.IGABB_ + IK_tot*1e3;        {muA/cm^2}   //05.06.2020 - added GABB   //16.10.2013 - factor was 200
  VW.ICl_ :=VW.ICl_leak_ + VW.IGABA_;                     {muA/cm^2}
  G_:=GlialBuffer(dt_);                   {mM/ms}
  { Extracellular diffusion }
  if If_Diffusion=1 then begin
     Diff_K :=Diff_coe*(IV.K_bath_of_x-IV.K) + D_2d_coe{mum^2/ms}*Lapl_K_{mM/mum2};   {mM/ms}
     Diff_Cl:=Diff_coe*(Clo_0-IV.Clo);                                                {mM/ms}
     Diff_Na:=Diff_coe*(Nao_0-IV.Nao);                                                {mM/ms}
  end else begin
     Diff_K:=0; Diff_Cl:=0; Diff_Na:=0;
  end;
  {*** KCC2 and NKCC1 *******************************************************}
  case KCC2_Item of
  0:   begin {Doyon KCC2 approximation}
         VW.IKCC2_:=Ikcc2_max*(IV.VK-IV.VCL)*1e3/((IV.VK-IV.VCL)*1e3+Vhalf);  {muA/cm^2}
       end;
  1:   begin {Cressman 2009; Wei 2014}
         VW.IKCC2_:=Ikcc2_max*(IV.VK-IV.VCl)/e0;  {muA/cm^2}
       end;
  end;
  VW.INKCC1_:=IV.Inkcc1_max_of_x{/(1+dexp(16-IV.K))}*(IV.VNa+IV.VK-2*IV.VCl)/e0;  {muA/cm^2} {[Dronne 2006], exp is in [Wei 2014], alternative - [Vera-Siguenza//BMB 2018]}
  if INCX_max>100 then begin { old model with tau_Ca=INCX_max }
     tau_Ca_:= INCX_max;
     IV.INaCaX:=-(0-IV.Ca)/tau_Ca_/gam;                                         {muA/cm^2}
  end else begin             { p.16, Fridlyand 2003 }
     IV.INaCaX:=INCX_max/(1+step(0.0015/IV.Ca,5))*(3*IV.VNa-IV.VCa-Umean)/e0;   {muA/cm^2}
  end;
  {*** Water ****************************************************************}
  v_:=IV.Vol;
  WaterInflux:=0; targetVol:=1; pio_pii:=0; dVol_dt:=0;
  if If_Volume<>0 then begin
     //pio_pii:=(IV.K-K_0) - (beta{/IV.Vol}+1)*(IV.Na-Na_0)
     //                    - (beta{/IV.Vol}+1)*(IV.Cl-Cl_0);                         {mM}
//     pio_pii:= IV.K-K_0    + IV.Clo-Clo_0 + IV.Nao-Nao_0
//                           -(IV.Cl -Cl_0  + IV.Na -Na_0);                        {mM}
     pio:=IV.K  + IV.Clo + IV.Nao + HCO3out/IV.Vol;                                {mM}
     pii:=IV.Ki + IV.Cl  + IV.Na  + HCO3in;                                        {mM}
     pio_pii:= pio - pii - pio_pii_0;                                              {mM}   // 19.02.2020  It was pio_pii:= pio - pio - pio_pii_0;
     targetVol:=1+0.1029*beta*(dexp(pio_pii/20)-1);                              {%}  {Wei 2014}
     //WaterInflux:=gam{mM*cm2/(muA*ms)}*beta*0.018{g/mmol}/1000{g/l}
     //            * (500{mol H2O/mol K}*IKCC2_[iP] + 590*INKCC1_[iP]);        {1/ms}
     //WaterInflux:=4.5{1/s}*(IV.K/(IV.K+4.5{mM}) - K_0/(K_0+4.5{mM}))/1000;       {1/ms}  {[Bai 2018]}
     case Vol_Item of
     0:  dVol_dt:=1/tau_Vol*(targetVol - v_)                  - WaterInflux;    {1/ms} {Wei}
     1:  dVol_dt:=                             Lw_Vol*pio_pii - WaterInflux;    {1/ms} {OstbyEinvoll}
     2:  dVol_dt:=1/tau_Vol*(1          -v_) + Lw_Vol*pio_pii - WaterInflux;    {1/ms} {Ostby&me}
     3:  dVol_dt:=(pio*v_/(pii+pio_pii_0) -v_)/dt_/1000;                        {1/ms} {steady osmos}
     end;
  end;
  {**************************************************************************}
  dKi_dt :=gam     *(-VW.IK_       - VW.IK_glu_
                   +2*IV.INaKpump + VW.IKCC2_ +   VW.INKCC1_);                  {mM/ms}
  dCl_dt :=gam     *( VW.ICl_     + VW.IKCC2_ + 2*VW.INKCC1_);                  {mM/ms}
  dNa_dt :=gam     *(-VW.INa_leak_ - VW.INa_glu_ + q_Na*nu + INa_ind*1e3             //INa_ind since 05.08.2020
                   -3*IV.INaKpump                + VW.INKCC1_+3*IV.INaCaX);    {mM/ms}
  dK_dt  :=-beta*dKi_dt + Diff_K  + G_{- dVol_dt/IV.Vol*IV.K};                   {mM/ms}
  dClo_dt:=-beta*dCl_dt + Diff_Cl     {- dVol_dt/IV.Vol*IV.Clo};                 {mM/ms}
  dNao_dt:=-beta*dNa_dt + Diff_Na     {- dVol_dt/IV.Vol*IV.Nao};                 {mM/ms}
  dO2_dt :=-gam*al_O2*IV.INaKpump + Diff_O2*(O2_bath-IV.O2);                     {mg/l/ms}
  dCa_dt :=gam     *(-VW.ICa_glu_/2 + q_Ca*nu/2                -  IV.INaCaX); {mM/ms}
  {**************************************************************************}
  IV.Vol:= IV.Vol + dt_*1000*dVol_dt;                                 {%}
  IV.Ki := IV.Ki  + dt_*1000*dKi_dt;                                  {mM}
  IV.K  :=(IV.K   + dt_*1000*dK_dt  )*v_/IV.Vol;                      {mM}
  IV.Cl := IV.Cl  + dt_*1000*dCl_dt;                                  {mM}
  IV.Clo:=(IV.Clo + dt_*1000*dClo_dt)*v_/IV.Vol;                      {mM}
  IV.Na := IV.Na  + dt_*1000*dNa_dt;                                  {mM}
  IV.Nao:=(IV.Nao + dt_*1000*dNao_dt)*v_/IV.Vol;                      {mM}
  IV.O2 := IV.O2  + dt_*1000*dO2_dt;                                  {mg/l}
  IV.Ca := IV.Ca  + dt_*1000*dCa_dt;                                  {mM}
  {**************************************************************************}
  if If_O2=0       then IV.O2:=O2_bath;
  if If_K_fixed=1  then IV.K :=K_fix;                              { K fixed, mM }
  if If_Cl_fixed=1 then begin IV.Cl:=Cl_fix; IV.Clo:=Clo_0; end;   { Cl fixed, mM }
  VW.I_K_Na_Cl_:=(dK_dt)/gam/beta; {muA/cm^2}
  IV.I_Glia:=G_   /gam/beta;          {muA/cm^2}
  IV.I_Diff:=Diff_K/gam/beta;         {muA/cm^2}
  IV.K :=max(IV.K, 1e-8);
  IV.Cl:=max(IV.Cl,1e-8);
  IV.Ca:=max(IV.Ca,1e-8);
  { Reversal potentials }
  ReversalsFromConcentrations;
  end;
end;
{******************************************************************************}

procedure TIons.Draw;
begin
  Form26.Visible:=true;
  Form26.Series1.AddXY(t*1e3,IV.VK*1e3);
  Form26.Series2.AddXY(t*1e3,IV.VGABA*1e3);
  Form26.Series3.AddXY(t*1e3,-2*IV.INaKpump);
  Form26.Series4.AddXY(t*1e3,VW.IK_);
  Form26.Series5.AddXY(t*1e3,IV.K);
  Form26.Series6.AddXY(t*1e3,IV.Cl);
  Form26.Series7.AddXY(t*1e3,VW.IK_leak_);
  Form26.Series8.AddXY(t*1e3,VW.ICl_leak_);
  Form26.Series9.AddXY(t*1e3,VW.IGABA_);
  Form26.Series10.AddXY(t*1e3,VW.IKCC2_);
  Form26.Series11.AddXY(t*1e3,IV.I_Glia+IV.I_Diff);
  Form26.Series12.AddXY(t*1e3,IV.VNa*1e3);
  Form26.Series13.AddXY(t*1e3,IV.Na);
  Form26.Series16.AddXY(t*1e3,IV.VL*1e3);
  Form26.Series17.AddXY(t*1e3,VW.IK_glu_);
  Form26.Series18.AddXY(t*1e3,VW.INa_glu_);
  Form26.Series19.AddXY(t*1e3,VW.INKCC1_);
  Form26.Series23.AddXY(t*1e3,IV.VCl*1e3);
  Form26.Series24.AddXY(t*1e3,IV.Vol*100);
  Form26.Series25.AddXY(t*1e3,IV.O2);
  Form26.Series26.AddXY(t*1e3,IV.Clo);
  Form26.Series27.AddXY(t*1e3,IV.Nao);
  Form26.Series30.AddXY(t*1e3,IV.Vol*100);
  Form26.Series31.AddXY(t*1e3,IV.Ki);
  Form26.Series33.AddXY(t*1e3,IV.Ca*1000);
  Form26.Series34.AddXY(t*1e3,IV.INaCaX);
end;

procedure TIons.Draw2;
begin
  Form26.Series14.AddXY(t*1e3,IV.VGABA*1e3);
  Form26.Series15.AddXY(t*1e3,IV.VNa*1e3);
  Form26.Series20.AddXY(t*1e3,IV.VL*1e3);
  Form26.Series21.AddXY(t*1e3,IV.Cl);
  Form26.Series22.AddXY(t*1e3,IV.Na);
  Form26.Series28.AddXY(t*1e3,IV.Clo);
  Form26.Series29.AddXY(t*1e3,IV.Nao);
  Form26.Series32.AddXY(t*1e3,IV.Ki);
end;

procedure TIons.InitiateWriting;
var FileName :string;
begin
  { Initiate writing }
  str(iP:1, FileName);
  FileName:='Ions_Pop'+FileName+'.dat';
  AssignFile(iii,FileName);  rewrite(iii);  CloseFile(iii);
end;

procedure TIons.Writing;
begin
  append(iii);
  {           1-A            2-B           3-C                      }
  write  (iii,t*1000:8:3,' ',IV.K:10:3,' ',IV.Cl:10:3,' ');
  {           4-D                   5-E                             }
  write  (iii,IV.VK*1e3:10:3,' ',   IV.VCl*1e3:10:3,' ');
  {           6-F                  7-G             8-H              }
  write  (iii,IV.INaKpump:10:3,' ',VW.IK_:10:3,' ',VW.IK_leak_:10:3,' ');
  {           9-I                      10-K                         }
  write  (iii,VW.ICl_leak_:10:3,' ',VW.IKCC2_:10:3,' ');
  {           11-K                      12-L                        }
  write  (iii,VW.I_K_Na_Cl_:10:3,' ',IV.I_Glia:10:3,' ');
  {           13-M                   14-N                           }
  write  (iii,VW.IK_glu_:10:3,' ',VW.INa_glu_:10:3,' ');
  {           15-O           16-P                                   }
  write  (iii,IV.Na:10:3,' ',IV.VNa*1e3:10:3,' ');
  {           17                        18                     19   }
  write  (iii,VW.IGABA_:10:3,' ',VW.INKCC1_:10:3,' ',IV.I_Diff:10:3,' ');
  {           20                    21                              }
  write  (iii,IV.VGABA*1e3:10:3,' ',IV.VL*1e3:10:3,' ');
  {           22             23              24                     }
  write  (iii,IV.O2:10:3,' ',IV.Vol:10:3,' ',IV.Ki:10:3,' ');
  {           25                  26                                }
  writeln(iii,IV.Ca*1000:10:3,' ',IV.INaCaX:10:3,' ');
  CloseFile(iii);
end;

end.


