unit SynCurrO;

interface
uses
  Classes,Forms,
  typeSynParsO,MathMyO,Unit4;

type
  {==================================================================}
  TSynCurr = class
  private
  protected
  public
    {Properties:}
    SP                                          :SynapseProperties;
    {Variables:}
    g,m,U,W,eM,UeM,WeM,xD,mD,q,Mem,MemU         :double;
    {Functions:}
    {*******************************************}
    procedure InitialConditions;
    function Sigmoid(x :double) :double;
    procedure Integrate(dt,mean_fNMDA_ :double);
    procedure EquateFrom(ASyn :TSynCurr);
    procedure EquateTo(ASyn :TSynCurr);

    constructor Create;
  end;
  {==================================================================}

implementation

constructor TSynCurr.Create;
begin
  inherited Create;
end;

procedure TSynCurr.EquateFrom(ASyn :TSynCurr);
var SP1  :SynapseProperties;
begin
  { Properties except pAxonRate}
  SP1:=SP;
  SP :=ASyn.SP ;
  SP.pAxonRate:=SP1.pAxonRate;
  { Variables }
  g  :=ASyn.g  ;
  m  :=ASyn.m  ;
  U  :=ASyn.U  ;
  W  :=ASyn.W  ;
  eM :=ASyn.eM ;
  UeM:=ASyn.UeM;
  WeM:=ASyn.WeM;
  xD :=ASyn.xD ;
  mD :=ASyn.mD;
  q  :=ASyn.q;
  Mem:=ASyn.Mem;
end;

procedure TSynCurr.EquateTo(ASyn :TSynCurr);
var SP1  :SynapseProperties;
begin
  { Properties except pAxonRate}
  SP1:=ASyn.SP;
  ASyn.SP :=SP ;
  ASyn.SP.pAxonRate:=SP1.pAxonRate;
  { Variables }
  ASyn.g  :=g  ;
  ASyn.m  :=m  ;
  ASyn.U  :=U  ;
  ASyn.W  :=W  ;
  ASyn.eM :=eM ;
  ASyn.UeM:=UeM;
  ASyn.WeM:=WeM;
  ASyn.xD :=xD ;
  ASyn.mD :=mD ;
  ASyn.q  :=q  ;
  ASyn.Mem:=Mem;
end;

procedure TSynCurr.InitialConditions;
begin
    m:=0;
    eM:=0;    UeM:=0;   WeM:=0;
    xD:=1;    Mem:=0;   MemU:=0;
    if SP.Name[1]+SP.Name[2]+SP.Name[3]='GAB' then xD:=Form4.DDSpinEdit129.Value
                                              else xD:=Form4.DDSpinEdit128.Value;
    mD:=1;
    q:=1;
    U:=m/SP.alpha;  W:=m/SP.beta;
    if abs(SP.alpha-SP.beta)<0.001 then U:=m;
end;

function TSynCurr.Sigmoid(x :double) :double;
begin
  if SP.IfApplySigmoidToAxonRate=1 then Sigmoid:=2/(1+dexp(-2*x))-1
                                   else Sigmoid:=x;
end;

procedure TSynCurr.Integrate(dt,mean_fNMDA_ :double);
var
  quant,mediator,c1,al,be,sc,tau_,nu_,q_,mxf_ :double;
begin
  nu_:=SP.pAxonRate^;
  quant:=0.001 {s};
  mediator:=Sigmoid(nu_ *quant)*(1-SP.IfBlock);                                 // number of excited synapses
  if SP.If_Saturates=1 then mediator:=mediator*max(1-m,0); //max - 18.03.2020   // number of synapses is finite
  { Extrasynaptic effects: kinetics depends on stimulus [B.Morales et al. 2002]
                           or mediator is accumulated and opens channels. }
  c1:=1;
  if SP.factor>0 then begin
        { Integrates extracellular mediator into 'ExtraMediat'. }
        al:=Form4.DDSpinEdit90.Value; {Hz}; be:=SP.beta;//5000 {Hz}; // changed on 6.02.2017
        //sc:=Max_K(al,be)*quant;
        sc:=1;                          { added on 31.01.2017 }
        //New_m(1,al,be,dt,mediator*xD/sc, eM,UeM,WeM);  // it was 1 instead of 0 before 31.01.2017; *xD introduced on 28.04.2018
        New_m6(1/al,dt,mediator*xD, eM);           // introduced on 30.04.2018
        { Extrasynaptic mediator changes synaptic kinetics }
        //c1:=1 + eM *SP.factor;       { commented on 31.01.2017 }
        { Extrasynaptic mediator affects similar to synaptic mediator }
        mediator:=mediator+eM*SP.factor;               // uncommented on 28.04.2018
  end;
  {* Non-dimensional conductance m ******************************************}
  New_m2(SP.IfInSyn_al_EQ_beta,SP.alpha/c1,SP.beta/c1,dt,mediator/quant, m,U);  // m accounts kinetics
  {**************************************************************************}
  {* Synaptic depression: dx/dt=(1-x)/tau - U*x*nu(t) [Loebel,Tsodyks 2002] *}
  {             or        dx/dt=(1-x)/tau - U*x*Sigm(nu(t)*quant)*(1-m)      }
  {  where      1/tau=Sigm(V)     [Zhou_JNsc2014_EAAT2_uptake_exchange.pdf]  }
  {             or 1/tau=Sigm(nu)                                            }
  tau_:=SP.tau_D0*mD;
  if SP.nu_thr_Depr<>0 then tau_:=tau_*dexp(nu_/SP.nu_thr_Depr);
  //q:=1; if m>0.1{01} {q>10}{Hz} then q:=0.01{0};  { Recovery only for silent synapses }
  //if SP.U_Depr>0 then  xD:=xD + dt*((1-xD)*q/tau_   - xD*m/tau_);

  if SP.U_Depr>0 then  q :=q  + dt*((1-q)/(10*tau_) - q *SP.U_Depr*nu_);
  if SP.U_Depr>0 then  q :=max(0,min(q ,1));
  if SP.U_Depr>0 then  xD:=xD + dt*((1-xD)*q/tau_   - xD*SP.U_Depr*nu_);
  if SP.U_Depr>0 then  xD:=max(0,min(xD,1));

  ////if SP.U_Depr>0 then New_m4(dt/tau_,1-xD*tau_*SP.U_Depr*nu_, xD,q);               // 04.06.2018
  { Frequency-dependent recovery, p.168 }
  if SP.aFDR>0   then  mD:=mD + dt*((1-mD)/SP.tauFDR - mD*SP.aFDR*nu_);         // 20.02.2018, see p.168
  if SP.aFDR>0   then  mD:=max(0,min(mD,1));
  { Memantine blockade of NMDA }
  if (SP.tau_Mem>0)and(SP.IfMem>0) then begin
      //Mem:=Mem + dt*(-Mem/SP.tau_Mem + m*SP.rate_Mem*(1-Mem));
      //Mem:=max(0,min(Mem,1));
      //New_m2(1,1/SP.tau_Mem,1/SP.tau_Mem,dt, SP.IfMem*m*xD*mean_fNMDA_*max(1-Mem,0), Mem,MemU);
      //New_m4(dt/SP.tau_Mem, SP.IfMem*mean_fNMDA_*max(1-Mem,0), Mem,MemU);
      mxf_:=m*xD*mean_fNMDA_;
      Mem:=Mem + dt*(-Mem{*mxf_} + mxf_*max(1-Mem,0)*SP.IfMem{Sigmoid(m*xD*SP.rate_Mem)})/SP.tau_Mem;
  end;
  {**************************************************************************}
  g:=SP.gmax*(m{+eM*SP.factor})*xD*(1-Mem);                   // 6.02.2017; eM commented on 28.04.2018
end;

end.
