unit LGN_stimulation;

interface
uses  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
      Dialogs, Math,
      LGN_Init;

procedure MovingGratings(t_ :double; i,j :integer; var pext :double);
procedure Bar(i,j :integer; var pext :double);
procedure Gradient(i,j :integer; var pext :double);
procedure Spot(i,j :integer; var pext :double);
procedure Ring(i,j :integer; var pext :double);
procedure ComplexStimulus(t_:double; i,j :integer; var pext :double);
procedure RF_with_squares(t_,dt_:double; i,j :integer; var pext :double);
procedure Shuffle_i_step(IfShuffle,N_steps :integer);
procedure Define_i_j_from_i_step(i_step_,ni,nj :integer; var i_,j_ :integer);

implementation
uses LGN_main,LGN_Unit1,LGN_Unit30,LGN_Lattice,MathMyO,LGN_Unit15,LGN_Unit33;


procedure MovingGratings(t_ :double; i,j :integer; var pext :double);
var Velocity,Length,Frequency,fi0,r_,l_,al,R_2,SpatPhase0   :double;
begin
  Frequency :=LGN_Form1.DDSpinEdit1.Value; {Hz}
  Length    :=LGN_Form1.DDSpinEdit2.Value{deg};
  fi0       :=LGN_Form1.DDSpinEdit5.Value{degrees}/180*pi+pi/2;
  SpatPhase0:=LGN_Form1.DDSpinEdit33.Value{degrees};
  if LGN_Form1.ComboBox1.ItemIndex=7 then begin
     R_2:=LGN_Form1.DDSpinEdit22.Value{deg};
  end else R_2:=888;
  Velocity :=Frequency*Length; {deg/s}
  r_:=sqrt(sqr((i-ni2_Screen)*dxy_LGN)+sqr((j-nj2_Screen)*dxy_LGN));
  al:=ArcTan2( (i-ni2_Screen)*dxy_LGN,     (j-nj2_Screen)*dxy_LGN);
  l_:=r_*cos(al+fi0);
  pext:=Background +
        AmplOfStimLum*cos((l_-Velocity*t_)/Length*2*pi + SpatPhase0/180*pi);
  if r_>R_2 then pext:=Background;
end;

procedure Bar(i,j :integer; var pext :double);
var Length,fi0,n_,l_,al,SpatPhase0   :double;
begin
  Length    :=LGN_Form1.DDSpinEdit2.Value{grad};
  fi0       :=LGN_Form1.DDSpinEdit5.Value{degrees}/180*pi+pi/2;
  SpatPhase0:=LGN_Form1.DDSpinEdit33.Value{degrees};                            //introduced on 26.08.2020
  { normal to the central line of the bar oriented with fi_0 }
  n_:=((i-ni2_Screen)*dxy_LGN-SpatPhase0)*cos(fi0)+(j-nj2_Screen)*dxy_LGN*sin(fi0);
  if abs(n_)>Length/2 then pext:=Background
                      else pext:=Background + AmplOfStimLum;
end;

procedure Gradient(i,j :integer; var pext :double);
var R,fi0,r_,n_,Lum   :double;
begin
  R:=LGN_Form1.DDSpinEdit2.Value; {grad.}
  fi0      :=LGN_Form1.DDSpinEdit5.Value{degrees}/180*pi;
  { distance from the center of the screen }
  r_:=sqrt(sqr((i-ni2_Screen)*dxy_LGN)+sqr((j-nj2_Screen)*dxy_LGN));
  { normal to the central line oriented with fi_0 }
  n_:=(i-ni2_Screen)*dxy_LGN*cos(fi0+pi/2)+(j-nj2_Screen)*dxy_LGN*sin(fi0+pi/2);
  { Luminance changing along the normal }
  Lum:=n_/R;//(n_/R+1)/2;
  if abs(r_)<=R then begin
                     pext:=Background + MeanStimLum + AmplOfStimLum*Lum;
                end else begin
                     pext:=Background;
                end;
end;

procedure Spot(i,j :integer; var pext :double);
var R,fi0,r_,n_,Lum   :double;
begin
  R:=LGN_Form1.DDSpinEdit2.Value; {grad.}
  { distance from the center of the screen }
  r_:=sqrt(sqr((i-ni2_Screen)*dxy_LGN)+sqr((j-nj2_Screen)*dxy_LGN));
  Lum:=AmplOfStimLum*dexp(-r_*r_/2/R/R);
  { Luminance outside the spot }
  if abs(r_)>R then pext:=Background
               else pext:=Background + {AmplOfStim}Lum;
end;

procedure Ring(i,j :integer; var pext :double);
var R1,R2,fi0,r_,n_,Lum   :double;
begin
  R1:=LGN_Form1.DDSpinEdit2.Value; {grad.}
  R2:=LGN_Form1.DDSpinEdit22.Value; {grad.}
  { distance from the center of the screen }
  r_:=sqrt(sqr((i-ni2_Screen)*dxy_LGN)+sqr((j-nj2_Screen)*dxy_LGN));
  { Luminance outside the spot }
  if abs(r_)<R1 then pext:=Background
                else if abs(r_)>R2 then pext:=Background
                                   else pext:=Background + AmplOfStimLum;
end;

procedure ComplexStimulus(t_:double; i,j :integer; var pext :double);
var R,delay,r_,SpeedOfSpot,EndOfFirstSpot,DurationOfSecondSpot,
    Width,Height,Shift,
    DurationOfThirdSpot,Lum,
    x_shift,y_shift,r_shift,Vx,Vy,x1_,x2_,y1_,x_,y_,dx_,dy_           :double;
    ni_First,nj_First,ni_Second,nj_Second,ni_Third,nj_Third           :integer;
begin
  case LGN_Form30.PageControl1.TabIndex of
  0: begin { Two/three spots }
       R                    :=LGN_Form30.DDSpinEdit16.Value; {grad.}
       delay                :=LGN_Form30.DDSpinEdit19.Value/1000; {s}
       EndOfFirstSpot       :=LGN_Form30.DDSpinEdit14.Value/1000; {s}
       DurationOfSecondSpot :=LGN_Form30.DDSpinEdit3.Value/1000; {s}
       DurationOfThirdSpot  :=LGN_Form30.DDSpinEdit21.Value/1000; {s}
       t_stim:=EndOfFirstSpot+delay+DurationOfSecondSpot;
       if DurationOfThirdSpot>0 then t_stim:=t_stim+delay+DurationOfThirdSpot;
       ni_First:=ni2_Screen;
       nj_First:=nj2_Screen;
       ni_Second:=ni_First  +trunc(LGN_Form30.DDSpinEdit1.Value/dxy_LGN);
       nj_Second:=nj_First  +trunc(LGN_Form30.DDSpinEdit2.Value/dxy_LGN);
       ni_Third :=ni_First+2*trunc(LGN_Form30.DDSpinEdit1.Value/dxy_LGN);
       nj_Third :=nj_First+2*trunc(LGN_Form30.DDSpinEdit2.Value/dxy_LGN);
       if t_<=EndOfFirstSpot then begin  { First spot }
          { distance from the center of the screen }
          r_:=sqrt(sqr((i-ni_First)*dxy_LGN)+sqr((j-nj_First)*dxy_LGN));
          Lum:=AmplOfStimLum*dexp(-r_*r_/2/R/R);
          { Luminance outside the spot }
          if abs(r_)>R then pext:=Background
                       else pext:=Background + {AmplOfStim}Lum;
       end else if (t_> EndOfFirstSpot+delay)and
                   (t_<=EndOfFirstSpot+delay+DurationOfSecondSpot) then begin
          { distance from the center of the screen }
          r_:=sqrt(sqr((i-ni_Second)*dxy_LGN)+sqr((j-nj_Second)*dxy_LGN));
          Lum:=AmplOfStimLum*dexp(-r_*r_/2/R/R);
          { Luminance outside the spot }
          if abs(r_)>R then pext:=Background
                       else pext:=Background + {AmplOfStim}Lum;
       end else if (t_> EndOfFirstSpot+delay+DurationOfSecondSpot+delay)and
                   (t_<=t_stim) then begin
          { distance from the center of the screen }
          r_:=sqrt(sqr((i-ni_Third )*dxy_LGN)+sqr((j-nj_Third )*dxy_LGN));
          Lum:=AmplOfStimLum*dexp(-r_*r_/2/R/R);
          { Luminance outside the spot }
          if abs(r_)>R then pext:=Background
                       else pext:=Background + {AmplOfStim}Lum;
       end else pext:=Background;
     end;
  1: begin { Moving spot }
       R                    :=LGN_Form30.DDSpinEdit17.Value; {grad.}
       SpeedOfSpot          :=LGN_Form30.DDSpinEdit4.Value; {deg/s}
       x_shift              :=LGN_Form30.DDSpinEdit18.Value;
       y_shift              :=LGN_Form30.DDSpinEdit20.Value;
       r_shift:=sqrt(sqr(x_shift)+sqr(y_shift));
       if r_shift>0 then begin Vx:=x_shift/r_shift; Vy:=y_shift/r_shift end else
                         begin Vx:=1;               Vy:=0; end;
       //t_stim:=t_end;                                                         //commented on 26.08.2020
       ni_First:=imin(ni2_Screen+trunc(t_*SpeedOfSpot/dxy_LGN*Vx), trunc(ni2_Screen+x_shift/dxy_LGN));
       nj_First:=imin(nj2_Screen+trunc(t_*SpeedOfSpot/dxy_LGN*Vy), trunc(nj2_Screen+y_shift/dxy_LGN));
       { distance from the center of the screen }
       r_:=sqrt(sqr((i-ni_First)*dxy_LGN)+sqr((j-nj_First)*dxy_LGN));
       Lum:=AmplOfStimLum*dexp(-r_*r_/2/R/R);
       { Luminance outside the spot }
       if abs(r_)>R then pext:=Background
                    else pext:=Background + {AmplOfStim}Lum;
     end;
  2: begin { Square and Line }
       R                    :=LGN_Form30.DDSpinEdit5.Value/2; {grad.}
       delay                :=LGN_Form30.DDSpinEdit13.Value/1000; {s}
       EndOfFirstSpot       :=LGN_Form30.DDSpinEdit15.Value/1000; {s}
       DurationOfSecondSpot :=LGN_Form30.DDSpinEdit8.Value/1000; {s}
       t_stim:=EndOfFirstSpot+delay+DurationOfSecondSpot;
       ni_First:=ni2_Screen;
       nj_First:=nj2_Screen;
       ni_Second:=ni_First+trunc(LGN_Form30.DDSpinEdit6.Value/dxy_LGN);
       if t_<EndOfFirstSpot then begin  { First spot }
          { distance from the center of the first }
          dx_:=(i-ni_First)*dxy_LGN;
          dy_:=(j-nj_First)*dxy_LGN;
          { Luminance inside the Line }
          if (abs(dx_)<=R)and(abs(dy_)<=R) then pext:=Background + AmplOfStimLum
                                           else pext:=Background;
       end else if t_>EndOfFirstSpot+delay then begin
          { distance from the end of the Line }
          x1_:=ni_First *dxy_LGN-R;
          x2_:=ni_Second*dxy_LGN+R;
          y1_:=nj_First *dxy_LGN;
          x_:=i*dxy_LGN;
          y_:=j*dxy_LGN;
          { Luminance inside the Line }
          if (x_>=x1_)and(x_<=x2_)and(abs(y_-y1_)<=R) then
                                                pext:=Background + AmplOfStimLum
                                           else pext:=Background;
       end else pext:=Background;
     end;
  3: begin { Growing Line }
       R                    :=LGN_Form30.DDSpinEdit12.Value/2; {grad.}
       t_stim               :=LGN_Form30.DDSpinEdit9.Value/1000; {s}
       ni_First:=ni2_Screen;
       nj_First:=nj2_Screen;
       ni_Second:=ni_First+trunc(LGN_Form30.DDSpinEdit7.Value/dxy_LGN);
       { distance from the end of the Line }
       x1_:=ni_First *dxy_LGN - R;
       x2_:=ni_First *dxy_LGN + (ni_Second-ni_First)/t_stim*t_*dxy_LGN + R;
       y1_:=nj_First *dxy_LGN;
       x_:=i*dxy_LGN;
       y_:=j*dxy_LGN;
       { Luminance inside the Line }
       if (x_>=x1_)and(x_<=x2_)and(abs(y_-y1_)<=R) then
                                                pext:=Background + AmplOfStimLum
                                           else pext:=Background;
     end;
  4: begin { Moving Square }
       Width                :=LGN_Form30.DDSpinEdit11.Value/2; {grad.}
       SpeedOfSpot          :=LGN_Form30.DDSpinEdit10.Value;   {deg/s}
       Height               :=LGN_Form30.DDSpinEdit24.Value/2; {grad.}
       Shift                :=LGN_Form30.DDSpinEdit25.Value;   {grad.}
       //t_stim:=t_end;                                                         //commented on 26.08.2020
       x1_:=ni2_Screen*dxy_LGN + t_*SpeedOfSpot + Shift;
       y1_:=nj2_Screen*dxy_LGN;
       { Luminance inside the Square }
       if (abs(i*dxy_LGN-x1_)<=Width)and(abs(j*dxy_LGN-y1_)<=Height) then
                                                pext:=Background + AmplOfStimLum
                                           else pext:=Background;
     end;
  end;
end;

{ Variables for "RF with squares" }
var
    i_sq,j_sq,n_sq :integer;
    R_sq,t_per_square,t_pause,x1_,y1_ :double;
    rrr,qqq :text;

procedure RF_with_squares(t_,dt_:double; i,j :integer; var pext :double);
{ Set stimulation by square-spots to reconstruct a Receptive Field }
var  n_sq_ :integer;
     x_,r_V1_,phi_V1_ :double;
begin
  { Initialize parameters }
  if (nt_LGN<=1)and(i=0)and(j=0) then begin
     n_sq:=-1;
     { Initiate randomization of squares }
     case LGN_Form30.ComboBox3.ItemIndex of
     0: Shuffle_i_step(1,ni*nj); // Shuffle square-spots on a rectangular grid of the screen
     1: Shuffle_i_step(1,ni);    // Shuffle square-spots on a horisontal line
     2: ;                        // Continuous line scaning
     end;
     t_per_square:=LGN_Form30.DDSpinEdit22.Value/1000; {s}
     t_pause     :=LGN_Form30.DDSpinEdit23.Value/1000; {s}
     R_sq        :=LGN_Form1.DDSpinEdit2.Value; {Radius_1, grad.}
     { Writing }
     assign(rrr,'RF_squares_t_x_y.dat'); rewrite(rrr); close(rrr);
     assign(qqq,'Activity_in_LGN_and_V1.dat'); rewrite(qqq);  close(qqq);
  end;
  n_sq_:=trunc((t_+1e-6)/(t_per_square+t_pause));
  { Set new square coordinates }
  if n_sq_<>n_sq then begin
     n_sq:=n_sq_;
     case LGN_Form30.ComboBox3.ItemIndex of
     0: begin   // Shuffle square-spots on a rectangular grid of the screen
          Define_i_j_from_i_step(n_sq+1,ni,nj, i_sq,j_sq);
        end;
     1: begin   // Shuffle square-spots on a horizontal line
          Define_i_j_from_i_step(n_sq+1,ni,nj, i_sq,j_sq);
          j_sq:=Diagram.j_RecNrn;
        end;
     2: begin   // Continuous line scaning at the cortical level of j_view
          i_sq:=n_sq+1;
          j_sq:=Diagram.j_RecNrn;
        end;
     end;
     x1_:=i_sq*dxy_LGN;
     y1_:=j_sq*dxy_LGN;
     { Writing }
     append(rrr);
     writeln(rrr,t_*1000:8:3,' ',x1_:8:3,' ',y1_:8:3);
     close(rrr);
  end;
  { Writing from one node }
  if (nt_LGN>1)and(i=Diagram.i_RecNrn)and(j=Diagram.j_RecNrn) then begin
      append(qqq);
      writeln(qqq,t_*1000:8:3,' ',zLGN[i,j]:8:3);
      close(qqq);
  end;
  { Luminance inside the Square }
  if ((abs(i*dxy_LGN-x1_)<=R_sq)and(abs(j*dxy_LGN-y1_)<=R_sq)) and
     (t_-n_sq*(t_per_square+t_pause)<t_per_square) then begin
      pext:=Background + AmplOfStimLum;
  end else begin
      pext:=MeanLum;
  end;
end;

{******************************************************************************}
{******************************************************************************}

{ Variables for "RF with squares" }
var
    list,listBack,i_step_Shuffled :array[0..10000] of integer;

procedure Shuffle_i_step(IfShuffle,N_steps :integer);
{ Compose a shuffled list of N_step integer numbers "i_step_Shuffled" }
var
    i,iR,iA :integer;

    function AgrumentOfList(i :integer) :integer;
    var j,A :integer;
    begin
      A:=list[0];
      if i>1 then begin
         for j:=1 to i-1 do begin
             A:=list[A];
         end;
      end;
      AgrumentOfList:=A;
    end;

begin
  if IfShuffle=1 then begin   { If Shuffle is on }
     { Inilial list }
     list[0]:=1;
     list[N_steps]:=1;
     listBack[1]:=N_steps;
     for i:=1 to N_steps-1 do begin
         list[i]:=i+1;
         listBack[i+1]:=i;
     end;
     for i:=0 to N_steps do i_step_Shuffled[i]:=0;
     Randomize;
     for i:=N_steps downto 1 do begin
         { Find random element in the list of survived elements }
         iR:=Random(i-1)+1;
         iA:=AgrumentOfList(iR);
         if iA=0 then begin
            iA:=AgrumentOfList(iR)+iA*0;
         end;
         i_step_Shuffled[i]:=iA;
         { Exclude element from the list }
         list[listBack[iA]]:=list[iA];
         listBack[list[iA]]:=listBack[iA];
         if list[0]=iA then list[0]:=list[iA];
         list[iA]:=0;
         listBack[iA]:=0;
     end;
  end else begin                             { If Shuffle is off }
     for i:=N_steps downto 1 do begin
         i_step_Shuffled[i]:=i;
     end;
  end;
end;

procedure Define_i_j_from_i_step(i_step_,ni,nj :integer; var i_,j_ :integer);
{ Find a point (i,j) on a grid ni x nj, according to the list "i_step_Shuffled". }
var i_step :integer;
begin
  i_step:=i_step_Shuffled[i_step_];
  j_:=trunc((i_step-1)/ni)+1;
  i_:=i_step-(j_-1)*ni;
end;

end.
