unit LGN_to_V1;

interface
uses  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
      Dialogs, Math,
      MathMyO,LGN_Init;

procedure PoissonEq_without_Symmetry(i,j:integer; sgmx,sgmy1,sgmy2,Back_:double;
                                     Pz_inp,Pz_out :Parr_X_Y);
function LGN_Convolve_with_ElongatedGaussian(r_V1,phi_V1,fi,Lag,charge :double) :double;

implementation
uses LGN_main;


procedure PoissonEq_without_Symmetry(i,j:integer; sgmx,sgmy1,sgmy2,Back_:double;
                                     Pz_inp,Pz_out :Parr_X_Y);
{  Integrating input rate WQ with exponential weight.
   Int exp(-r) dx dy = 2*pi, Int exp(-r^2) dx dy = pi }
var
    ii,jj,degree_,i_left,i_right,j_bottom,j_top  :integer;
    x,y,xx,yy,S,d_,Norm,W,sgm_max_               :double;
begin
  degree_:=2;
  x:=i*dxy_LGN;  y:=j*dxy_LGN;
  sgm_max_:=max(max(sgmx,sgmy1),sgmy2);
  i_left  :=trunc((x-2*sgm_max_)/dxy_LGN);
  i_right :=trunc((x+2*sgm_max_)/dxy_LGN);
  j_bottom:=trunc((y-2*sgm_max_)/dxy_LGN);
  j_top   :=trunc((y+2*sgm_max_)/dxy_LGN);
  S:=0;  Norm:=0;
  for ii:=imax(0,i_left) to imin(ni+1,i_right) do begin
      for jj:=imax(0,j_bottom) to imin(nj+1,j_top) do begin
          xx:=ii*dxy_LGN;
          yy:=jj*dxy_LGN;
          if yy<y then d_:=sqrt(sqr((x-xx)/sgmx)+sqr((y-yy)/sgmy1))
                  else d_:=sqrt(sqr((x-xx)/sgmx)+sqr((y-yy)/sgmy2));
          if degree_=2 then d_:=sqr(d_);
          {*************************************************************}
          S   :=S    +  (Pz_inp^[ii,jj]-Back_)*dexp(-d_)*dxy_LGN*dxy_LGN;
          {*************************************************************}
          Norm:=Norm +                         dexp(-d_)*dxy_LGN*dxy_LGN;
      end;
  end;
                     W:=2*pi*sgmx*(sgmy1+sgmy2)/2;
  if degree_=2 then  W:=  pi*sgmx*(sgmy1+sgmy2)/2;
  Pz_out^[i,j]:=Back_+S/Norm;
end;

{********************************************************************************************}
function LGN_Convolve_with_RoundGaussian(r_V1,phi_V1,fi,charge :double; Pz :Parr_X_Y) :double;
{  Integrating input rate WQ with exponential weight for a cortical cell with
   RF with the polar coordinates in V1: "r_V1", "phi_V1"
   with shift "Off_shift" along big axis,
   doubled preferred orientation "fi"
   and RF size equal to min("sgm_pref","sgm_orth").
}
var
    ii,jj,i_left,i_right,j_bottom,j_top                       :integer;
    xx,yy,S,d_,Norm,cos_,sin_,xRF,yRF,z_,sgm_,xx_Off          :double;
begin
  { Round RF }
  sgm_:=min(sgm_pref,sgm_orth);                          { ! }
  sin_:=sin(fi/2+pi/2);
  cos_:=cos(fi/2+pi/2);
  Translate_r_phi_V1_to_x_y_LGN(r_V1,phi_V1, xRF,yRF);
  { Shift RF center along big RF-axis }
  xx_Off:=-Off_shift*charge;                             { ! }
  xRF:=xRF + xx_Off*cos_;
  yRF:=yRF + xx_Off*sin_;
  { Define square embedding RF }
  i_left  :=trunc((xRF-2*sgm_)/dxy_LGN);
  i_right :=trunc((xRF+2*sgm_)/dxy_LGN);
  j_bottom:=trunc((yRF-2*sgm_)/dxy_LGN);
  j_top   :=trunc((yRF+2*sgm_)/dxy_LGN);
  { Integrate over RF }
  S:=0;  Norm:=0;
  for ii:=imax(0,i_left) to imin(ni+1,i_right) do begin
      for jj:=imax(0,j_bottom) to imin(nj+1,j_top) do begin
          { Rotation matrix ( cos, sin )
                            (-sin, cos )   }
          xx:= (ii*dxy_LGN-xRF)*cos_ + (jj*dxy_LGN-yRF)*sin_; //along big   RF-axis
          yy:=-(ii*dxy_LGN-xRF)*sin_ + (jj*dxy_LGN-yRF)*cos_; //along small RF-axis
          { Round RF }
          d_:=sqr(xx/sgm_)+sqr(yy/sgm_);
          { Off- LGN-cells contribute }
          z_:=Pz^[ii,jj];
          {**************************************}
          S   :=S  +  z_*exp(-d_)*dxy_LGN*dxy_LGN;
          {**************************************}
      end;
  end;
  Norm:=pi*sgm_*sgm_;
  LGN_Convolve_with_RoundGaussian:=S/Norm;
end;

{**************************************************************************************}
function LGN_Convolve_with_ElongatedGaussian(r_V1,phi_V1,fi,Lag,charge :double) :double;
{**************************************************************************************}
{  Integrating input rate WQ with exponential weight for a cortical cell with
   RF with the polar coordinates in V1: "r_V1", "phi_V1",
   doubled preferred orientation "fi"
   and elongations "sgm_pref" and "sgm_orth".
   DS is provided by "Lag".
}
var
    ii,jj,i_left,i_right,j_bottom,j_top                          :integer;
    xx,yy,S,d_,Norm,cos_,sin_,xRF,yRF,delay_,z_,RF_On,RF_Off     :double;
begin
  sin_:=sin(fi/2+pi/2);
  cos_:=cos(fi/2+pi/2);
  Translate_r_phi_V1_to_x_y_LGN(r_V1,phi_V1, xRF,yRF);
  i_left  :=trunc((xRF-2*max(sgm_pref,sgm_orth))/dxy_LGN);
  i_right :=trunc((xRF+2*max(sgm_pref,sgm_orth))/dxy_LGN);
  j_bottom:=trunc((yRF-2*max(sgm_pref,sgm_orth))/dxy_LGN);
  j_top   :=trunc((yRF+2*max(sgm_pref,sgm_orth))/dxy_LGN);
  { Integrate over RF }
  S:=0;  Norm:=0;
  for ii:=imax(0,i_left) to imin(ni+1,i_right) do begin
      for jj:=imax(0,j_bottom) to imin(nj+1,j_top) do begin
          { Rotation matrix ( cos, sin )
                            (-sin, cos )   }
          xx:= (ii*dxy_LGN-xRF)*cos_ + (jj*dxy_LGN-yRF)*sin_; //along big   RF-axis
          yy:=-(ii*dxy_LGN-xRF)*sin_ + (jj*dxy_LGN-yRF)*cos_; //along small RF-axis
          { Ellipsoidal RF }
          d_:=sqr(xx/sgm_pref)+sqr(yy/sgm_orth);
          if xx*charge<0 then begin                                             //it was yy before 13.01.2016
             { On-, lagged    (sustained) LGN-cells  contribution }
             case DS_model of
             0,1:  z_:=max(0,zLGN[ii,jj] - Lag*dzLGN_dt[ii,jj]);  { None or Shift Off around On }              //max was inserted on 20.01.2016
             2,3:  z_:=max(0,zLGN_Sust[ii,jj]);
             4:    z_:=max(0,zLGN_Off[ii,jj]);       { Ellipse On/Off }       //inserted on 17.01.2020
             5:    z_:=max(0,zLGN_Off[ii,jj]);       { Shift On around Off }
             end;
          end else begin
             { On-, nonlagged (transient) LGN-cells  contribution }
             case DS_model of
             0,1:  z_:=max(0,zLGN[ii,jj]);           { None or Shift Off around On }                           //max was inserted on 20.01.2016
             2:    z_:=max(0,zLGN[ii,jj]);                   { Transient=On }
             3:    z_:=max(0,zLGN[ii,jj]+zLGN_Off[ii,jj]){/2}; { Transient=On+Off }  // 1/2 is deleted on 06.03.2019
             4:    z_:=max(0,zLGN[ii,jj]);           { Ellipse On/Off }      //inserted on 17.01.2020
             5:    z_:=max(0,zLGN_Off[ii,jj]);       { Shift On around Off }
             end;
          end;
          {****************************************}
          S   :=S    +  z_*exp(-d_)*dxy_LGN*dxy_LGN;
          {****************************************}
          //Norm:=Norm + dexp(-d_)*dx*dy;
      end;
  end;
  Norm:=pi*sgm_pref*sgm_orth;
  {**********************************************************************}
  { Summarize contributions of On and Off-cells, Transient and Sustained }
  case DS_model of
  0,2,3,4:    // None or Lag/Nonlag or
              // Transient/Sustained  or Ellipse On/Off
       LGN_Convolve_with_ElongatedGaussian:=S/Norm;
  1: begin  // Shift Off around On
       RF_On :=pi*sgm_pref*sgm_orth;
       RF_Off:=pi*sqr(min(sgm_pref,sgm_orth));
       LGN_Convolve_with_ElongatedGaussian:=
         (S + LGN_Convolve_with_RoundGaussian(r_V1,phi_V1,fi,charge,@zLGN_Off)*RF_Off)
         /(RF_On+RF_Off);
     end;
  5: begin  // Shift On around Off
       RF_Off :=pi*sgm_pref*sgm_orth;
       RF_On:=pi*sqr(min(sgm_pref,sgm_orth));
       LGN_Convolve_with_ElongatedGaussian:=
         (S + LGN_Convolve_with_RoundGaussian(r_V1,phi_V1,fi,charge,@zLGN    )*RF_On)
         /(RF_On+RF_Off);
     end;
  end;
end;
{******************************************************************************}

end.
